{%- extends "basic/layout.html" %}
{%- block relbar2 %}{% endblock %}
{%- block footer %}
    <div class="footer">
      &copy; Copyright {{ copyright }}.
      Created using <a href="http://sphinx.pocoo.org/">Sphinx</a>.
    </div>
{%- endblock %}
