Jinja2 Documentation
--------------------

.. toctree::
   :maxdepth: 2

   intro
   api
   sandbox
   templates
   extensions
   integration
   switching
   tricks

Additional Information
----------------------

.. toctree::
   :maxdepth: 2

   faq
   changelog

If you can't find the information you're looking for, have a look at the
index or try to find it using the search function:

* :ref:`genindex`
* :ref:`search`
